@@ -6,9 +6,9 @@ from . import X86ElfPicPltResolver
 from . import JumpTableResolver
 from . import X86PeIatResolver
 from . import AMD64ElfGotResolver
+from . import ConstantResolver
 from . import ArmElfFastResolver
 
-
 DEFAULT_RESOLVERS = {
     'X86': {
         cle.MetaELF: [ X86ElfPicPltResolver, ],
@@ -32,7 +32,7 @@ DEFAULT_RESOLVERS = {
     'ARMCortexM': {
         cle.MetaELF: [ ArmElfFastResolver, ]
     },
-    'ALL': [ JumpTableResolver ],
+    'ALL': [ ConstantResolver, JumpTableResolver ],
 }
 
 
@@ -41,7 +41,7 @@ def default_indirect_jump_resolvers(obj, project):
     resolvers = [ ]
     for k, lst in arch_specific.items():
         if isinstance(obj, k):
-            resolvers = lst
+            resolvers = list(lst)
             break
 
     resolvers += DEFAULT_RESOLVERS['ALL']
