@@ -5,10 +5,9 @@ from ailment import Block
 from ailment.statement import Call, Statement, ConditionalJump, Assignment, Store, Return
 from ailment.expression import Load, Expression, BinaryOp, UnaryOp, Convert, ITE, DirtyExpression, VEXCCallExpression
 
-
-class AILBlockWalker:
+class AILBlockWalkerBase:
     """
-    Walks all statements and expressions of an AIL node.
+    Walks all statements and expressions of an AIL node and do nothing.
     """
     def __init__(self, stmt_handlers=None, expr_handlers=None):
 
@@ -58,6 +57,96 @@ class AILBlockWalker:
             return handler(stmt_idx, stmt, block)
         return None
 
+    def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement],
+                     block: Optional[Block]) -> Any:
+        try:
+            handler = self.expr_handlers[type(expr)]
+        except KeyError:
+            handler = None
+
+        if handler:
+            return handler(expr_idx, expr, stmt_idx, stmt, block)
+        return None
+
+    #
+    # Default handlers
+    #
+
+    def _handle_Assignment(self, stmt_idx: int, stmt: Assignment, block: Optional[Block]):
+        self._handle_expr(0, stmt.dst, stmt_idx, stmt, block)
+        self._handle_expr(1, stmt.src, stmt_idx, stmt, block)
+
+    def _handle_Call(self, stmt_idx: int, stmt: Call, block: Optional[Block]):
+        if stmt.args:
+            for i, arg in enumerate(stmt.args):
+                self._handle_expr(i, arg, stmt_idx, stmt, block)
+
+    def _handle_Store(self, stmt_idx: int, stmt: Store, block: Optional[Block]):
+        self._handle_expr(0, stmt.addr, stmt_idx, stmt, block)
+        self._handle_expr(1, stmt.data, stmt_idx, stmt, block)
+
+    def _handle_ConditionalJump(self, stmt_idx: int, stmt: ConditionalJump, block: Optional[Block]):
+        self._handle_expr(0, stmt.condition, stmt_idx, stmt, block)
+        self._handle_expr(1, stmt.true_target, stmt_idx, stmt, block)
+        self._handle_expr(2, stmt.false_target, stmt_idx, stmt, block)
+
+    def _handle_Return(self, stmt_idx: int, stmt: Return, block: Optional[Block]):
+        if stmt.ret_exprs:
+            for i, ret_expr in enumerate(stmt.ret_exprs):
+                self._handle_expr(i, ret_expr, stmt_idx, stmt, block)
+
+    def _handle_Load(self, expr_idx: int, expr: Load, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        self._handle_expr(0, expr.addr, stmt_idx, stmt, block)
+
+    def _handle_CallExpr(self, expr_idx: int, expr: Call, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        if expr.args:
+            for i, arg in enumerate(expr.args):
+                self._handle_expr(i, arg, stmt_idx, stmt, block)
+
+    def _handle_BinaryOp(self, expr_idx: int, expr: BinaryOp, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        self._handle_expr(0, expr.operands[0], stmt_idx, stmt, block)
+        self._handle_expr(1, expr.operands[1], stmt_idx, stmt, block)
+
+    def _handle_UnaryOp(self, expr_idx: int, expr: UnaryOp, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        self._handle_expr(0, expr.operand, stmt_idx, stmt, block)
+
+    def _handle_Convert(self, expr_idx: int, expr: Convert, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        self._handle_expr(expr_idx, expr.operand, stmt_idx, stmt, block)
+
+    def _handle_ITE(self, expr_idx: int, expr: ITE, stmt_idx: int, stmt: Statement, block: Optional[Block]):
+        self._handle_expr(0, expr.cond, stmt_idx, stmt, block)
+        self._handle_expr(1, expr.iftrue, stmt_idx, stmt, block)
+        self._handle_expr(2, expr.iffalse, stmt_idx, stmt, block)
+
+    def _handle_DirtyExpression(self, expr_idx: int, expr: DirtyExpression, stmt_idx: int, stmt: Statement,
+                                block: Optional[Block]):
+        self._handle_expr(0, expr.dirty_expr, stmt_idx, stmt, block)
+
+    def _handle_VEXCCallExpression(self, expr_idx: int, expr: VEXCCallExpression, stmt_idx: int, stmt: Statement,
+                                   block: Optional[Block]):
+        for idx, operand in enumerate(expr.operands):
+            self._handle_expr(idx, operand, stmt_idx, stmt, block)
+
+
+class AILBlockWalker(AILBlockWalkerBase):
+    """
+    Walks all statements and expressions of an AIL node, and rebuilds expressions, statements, or blocks if needed.
+
+    If you need a pure walker without rebuilding, use AILBlockWalkerBase instead.
+    """
+    def __init__(self, stmt_handlers=None, expr_handlers=None):
+        super().__init__(stmt_handlers=stmt_handlers, expr_handlers=expr_handlers)
+
+    def _handle_stmt(self, stmt_idx: int, stmt: Statement, block: Optional[Block]) -> Any:
+        try:
+            handler = self.stmt_handlers[type(stmt)]
+        except KeyError:
+            handler = None
+
+        if handler:
+            return handler(stmt_idx, stmt, block)
+        return None
+
     def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement],
                      block: Optional[Block]) -> Any:
         try:
@@ -173,10 +262,21 @@ class AILBlockWalker:
     def _handle_Return(self, stmt_idx: int, stmt: Return, block: Optional[Block]):
         if stmt.ret_exprs:
             i = 0
+            changed = False
+            new_ret_exprs = [None] * len(stmt.ret_exprs)
             while i < len(stmt.ret_exprs):
-                self._handle_expr(i, stmt.ret_exprs[i], stmt_idx, stmt, block)
+                new_ret_expr = self._handle_expr(i, stmt.ret_exprs[i], stmt_idx, stmt, block)
+                if new_ret_expr is not None:
+                    new_ret_exprs[i] = new_ret_expr
+                    changed = True
+                else:
+                    new_ret_exprs[i] = stmt.ret_exprs[i]
                 i += 1
 
+            if changed:
+                new_stmt = Return(stmt.idx, stmt.target, new_ret_exprs, **stmt.tags)
+                block.statements[stmt_idx] = new_stmt
+
     def _handle_Load(self, expr_idx: int, expr: Load, stmt_idx: int, stmt: Statement, block: Optional[Block]):
         addr = self._handle_expr(0, expr.addr, stmt_idx, stmt, block)
 
@@ -232,6 +332,7 @@ class AILBlockWalker:
         if changed:
             new_expr = expr.copy()
             new_expr.operands = (operand_0, operand_1)
+            new_expr.depth = max(operand_0.depth, operand_1.depth) + 1
             return new_expr
         return None
 
@@ -264,7 +365,7 @@ class AILBlockWalker:
         else:
             iftrue = expr.iftrue
 
-        iffalse = self._handle_expr(1, expr.iffalse, stmt_idx, stmt, block)
+        iffalse = self._handle_expr(2, expr.iffalse, stmt_idx, stmt, block)
         if iffalse is not None and iffalse is not expr.iffalse:
             changed = True
         else:
