@@ -2,19 +2,24 @@ import logging
 
 import ailment
 
-from ... import AnalysesHub
-from .optimization_pass import OptimizationPass
+from .optimization_pass import OptimizationPass, OptimizationPassStage
 
 _l = logging.getLogger(name=__name__)
 
 
 class BasePointerSaveSimplifier(OptimizationPass):
+    """
+    Removes the effects of base pointer stack storage at function invocation and restoring at function return.
+    """
 
-    ARCHES = ['X86', 'AMD64', 'ARMEL', "ARMCortexM"]
+    ARCHES = ['X86', 'AMD64', 'ARMEL', 'ARMHF', "ARMCortexM", "MIPS32", "MIPS64"]
     PLATFORMS = ["cgc", 'linux']
+    STAGE = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION
+    NAME = "Simplify base pointer saving"
+    DESCRIPTION = __doc__.strip()
 
-    def __init__(self, func, blocks, graph):
-        super().__init__(func, blocks=blocks, graph=graph)
+    def __init__(self, func, **kwargs):
+        super().__init__(func, **kwargs)
         self.analyze()
 
     def _check(self):
@@ -115,32 +120,22 @@ class BasePointerSaveSimplifier(OptimizationPass):
         baseptr_restore_stmts = [ ]
 
         for endpoint in endpoints:
-            endpoint_block = self._get_block(endpoint.addr)
-
-            if endpoint_block is None:
-                # the block is not found
-                _l.debug("Unexpected: Function endpoint %#x is not found. Maybe it has been removed by other "
-                         "optimization passes", endpoint.addr)
-                continue
-
-            for idx, stmt in enumerate(endpoint_block.statements):
-                if isinstance(stmt, ailment.Stmt.Assignment) \
-                        and isinstance(stmt.dst, ailment.Expr.Register) \
-                        and stmt.dst.reg_offset == self.project.arch.bp_offset \
-                        and isinstance(stmt.src, ailment.Expr.Load) \
-                        and isinstance(stmt.src.addr, ailment.Expr.StackBaseOffset):
-                    baseptr_restore_stmts.append((endpoint_block, idx, stmt.src.addr))
-                    break
-            else:
-                if endpoint.addr not in callouts_and_jumpouts:
-                    _l.debug("Could not find baseptr restoring statement in function %#x.", endpoint.addr)
-                    return None
+            for endpoint_block in self._get_blocks(endpoint.addr):
+                for idx, stmt in enumerate(endpoint_block.statements):
+                    if isinstance(stmt, ailment.Stmt.Assignment) \
+                            and isinstance(stmt.dst, ailment.Expr.Register) \
+                            and stmt.dst.reg_offset == self.project.arch.bp_offset \
+                            and isinstance(stmt.src, ailment.Expr.Load) \
+                            and isinstance(stmt.src.addr, ailment.Expr.StackBaseOffset):
+                        baseptr_restore_stmts.append((endpoint_block, idx, stmt.src.addr))
+                        break
                 else:
-                    _l.debug("No baseptr restoring statement is found at callout/jumpout site %#x. Might be expected.",
-                             endpoint.addr
-                             )
+                    if endpoint.addr not in callouts_and_jumpouts:
+                        _l.debug("Could not find baseptr restoring statement in function %#x.", endpoint.addr)
+                        return None
+                    else:
+                        _l.debug("No baseptr restoring statement is found at callout/jumpout site %#x. Might be expected.",
+                                 endpoint.addr
+                                 )
 
         return baseptr_restore_stmts
-
-
-AnalysesHub.register_default('BasePointerSaveSimplifier', BasePointerSaveSimplifier)
