@@ -2,13 +2,15 @@ import logging
 
 from ailment import Expr
 
-from ... import AnalysesHub
 from .engine_base import SimplifierAILEngine, SimplifierAILState
 from .optimization_pass import OptimizationPass, OptimizationPassStage
 
 _l = logging.getLogger(name=__name__)
 
 class MultiSimplifierAILEngine(SimplifierAILEngine):
+    """
+    An AIL pass for the multi simplifier
+    """
 
     def _ail_handle_Add(self, expr):
 
@@ -25,31 +27,31 @@ class MultiSimplifierAILEngine(SimplifierAILEngine):
         # 2*x + x = 3*x
         if Expr.BinaryOp in [type(operand_0), type(operand_1)]:
             if isinstance(operand_1, Expr.BinaryOp) and operand_1.op == 'Mul' and \
-                (not isinstance(operand_0, Expr.BinaryOp) or
-                    (isinstance(operand_0, Expr.BinaryOp) and operand_0.op != 'Mul')):
-                    x0 = operand_0
-                    x1_index = 0 if isinstance(operand_1.operands[1], Expr.Const) else 1
-                    x1 = operand_1.operands[x1_index]
-                    const_x1 = operand_1.operands[1-x1_index]
-                    if x0 == x1:
-                        new_const = Expr.Const(const_x1.idx, None, const_x1.value+1, const_x1.bits)
-                        new_expr = Expr.BinaryOp(expr.idx, 'Mul', [x0, new_const], expr.signed, **expr.tags)
-                        return new_expr
+                    (not isinstance(operand_0, Expr.BinaryOp) or
+                        (isinstance(operand_0, Expr.BinaryOp) and operand_0.op != 'Mul')):
+                x0 = operand_0
+                x1_index = 0 if isinstance(operand_1.operands[1], Expr.Const) else 1
+                x1 = operand_1.operands[x1_index]
+                const_x1 = operand_1.operands[1-x1_index]
+                if x0 == x1:
+                    new_const = Expr.Const(const_x1.idx, None, const_x1.value+1, const_x1.bits)
+                    new_expr = Expr.BinaryOp(expr.idx, 'Mul', [x0, new_const], expr.signed, **expr.tags)
+                    return new_expr
             elif isinstance(operand_0, Expr.BinaryOp) and operand_0.op == 'Mul' and \
-                (not isinstance(operand_1, Expr.BinaryOp) or
-                    (isinstance(operand_1, Expr.BinaryOp) and operand_1.op != 'Mul')):
+                    (not isinstance(operand_1, Expr.BinaryOp) or
+                        (isinstance(operand_1, Expr.BinaryOp) and operand_1.op != 'Mul')):
 
-                    x1 = operand_1
-                    x0_index = 0 if isinstance(operand_0.operands[1], Expr.Const) else 1
-                    x0 = operand_0.operands[x0_index]
-                    const_x0 = operand_0.operands[1-x0_index]
-                    if x0 == x1:
-                        new_const = Expr.Const(const_x0.idx, None, const_x0.value+1, const_x0.bits)
-                        new_expr = Expr.BinaryOp(expr.idx, 'Mul', [x1, new_const], expr.signed, **expr.tags)
-                        return new_expr
+                x1 = operand_1
+                x0_index = 0 if isinstance(operand_0.operands[1], Expr.Const) else 1
+                x0 = operand_0.operands[x0_index]
+                const_x0 = operand_0.operands[1-x0_index]
+                if x0 == x1:
+                    new_const = Expr.Const(const_x0.idx, None, const_x0.value+1, const_x0.bits)
+                    new_expr = Expr.BinaryOp(expr.idx, 'Mul', [x1, new_const], expr.signed, **expr.tags)
+                    return new_expr
             # 2*x + 3*x = 5*x
             elif isinstance(operand_0, Expr.BinaryOp) and isinstance(operand_1, Expr.BinaryOp) and \
-                operand_0.op == 'Mul' and operand_1.op == 'Mul':
+                    operand_0.op == 'Mul' and operand_1.op == 'Mul':
                 if Expr.Const in [type(operand_0.operands[0]), type(operand_0.operands[1])] \
                     and Expr.Const in [type(operand_1.operands[0]), type(operand_1.operands[1])]:
                     x0_index = 0 if isinstance(operand_0.operands[1], Expr.Const) else 1
@@ -151,7 +153,10 @@ class MultiSimplifierAILEngine(SimplifierAILEngine):
             if Expr.BinaryOp in [type(operand_0), type(operand_1)]:
                 const_, x0 = (operand_0, operand_1) if isinstance(operand_0, Expr.Const) else (operand_1, operand_0)
                 if x0.op == 'Mul' and Expr.Const in [type(x0.operands[0]), type(x0.operands[1])]:
-                    const_x0, x = (x0.operands[0], x0.operands[1]) if isinstance(x0.operands[0], Expr.Const) else (x0.operands[1], x0.operands[0])
+                    if isinstance(x0.operands[0], Expr.Const):
+                        const_x0, x = x0.operands[0], x0.operands[1]
+                    else:
+                        const_x0, x = x0.operands[1], x0.operands[0]
                     new_const = Expr.Const(const_.idx, None, const_.value*const_x0.value, const_.bits)
                     new_expr = Expr.BinaryOp(expr.idx, 'Mul', [x, new_const], expr.signed, **expr.tags)
                     return new_expr
@@ -170,10 +175,15 @@ class MultiSimplifierAILEngine(SimplifierAILEngine):
         return expr
 
 class MultiSimplifier(OptimizationPass):
+    """
+    Implements several different arithmetic optimizations.
+    """
 
     ARCHES = ["X86", "AMD64"]
     PLATFORMS = ["linux", "windows"]
     STAGE = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION
+    NAME = "Simplify various arithmetic expressions"
+    DESCRIPTION = __doc__.strip()
 
     def __init__(self, func, **kwargs):
 
@@ -199,5 +209,3 @@ class MultiSimplifier(OptimizationPass):
                 _l.debug("new block: %s", new_block.statements)
 
             self._update_block(block, new_block)
-
-AnalysesHub.register_default("MultiSimplifier", MultiSimplifier)
