@@ -1,13 +1,15 @@
-import archinfo
 from ailment.expression import BinaryOp, Const, Load
 
 from .base import PeepholeOptimizationExprBase
 
 
 class SimplifyPcRelativeLoads(PeepholeOptimizationExprBase):
+    """
+    Simplifying pc-relative loads.
+    """
     __slots__ = ()
 
-    name = "Simplify PC-relative loads"
+    NAME = "Simplify PC-relative loads"
     expr_classes = (BinaryOp, )
 
     def optimize(self, expr: BinaryOp):
@@ -32,26 +34,3 @@ class SimplifyPcRelativeLoads(PeepholeOptimizationExprBase):
                         return Const(None, None, value, self.project.arch.bits, **expr.tags)
 
         return expr
-
-    def _is_pc(self, pc, addr) -> bool:
-        if archinfo.arch_arm.is_arm_arch(self.project.arch):
-            if pc & 1 == 1:
-                # thumb mode
-                pc = pc - 1
-                return addr == pc + 4
-            else:
-                # arm mode
-                return addr == pc + 8
-        return pc == addr
-
-    def _is_in_readonly_section(self, addr: int) -> bool:
-        sec = self.project.loader.find_section_containing(addr)
-        if sec is not None:
-            return not sec.is_writable
-        return False
-
-    def _is_in_readonly_segment(self, addr: int) -> bool:
-        seg = self.project.loader.find_segment_containing(addr)
-        if seg is not None:
-            return not seg.is_writable
-        return False
