@@ -9,7 +9,8 @@ class SimConcretizationStrategyRange(SimConcretizationStrategy):
         super(SimConcretizationStrategyRange, self).__init__(**kwargs)
         self._limit = limit
 
-    def _concretize(self, memory, addr):
-        mn,mx = self._range(memory, addr)
+    def _concretize(self, memory, addr, **kwargs):
+        mn,mx = self._range(memory, addr, **kwargs)
         if mx - mn <= self._limit:
-            return self._eval(memory, addr, self._limit)
+            return self._eval(memory, addr, self._limit, **kwargs)
+        return None
