@@ -28,6 +28,28 @@ class SimEngineUnicorn(SuccessorsMixin):
     - extra_stop_points:   A collection of addresses at which execution should halt
     """
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # Cache of details of basic blocks containing statements that need to re-executed
+        self._block_details_cache = {}
+        # Addresses of basic blocks which native interface will not execute
+        self._stop_block_addrs_cache = set()
+        # Stop reasons to track and not switch to native interface for those basic blocks
+        self._stop_reasons_to_track = STOP.unsupported_reasons | \
+            {STOP.STOP_STOPPOINT, STOP.STOP_ERROR, STOP.STOP_NODECODE, STOP.STOP_SYSCALL, STOP.STOP_EXECNONE,
+             STOP.STOP_ZEROPAGE, STOP.STOP_NOSTART, STOP.STOP_SEGFAULT, STOP.STOP_ZERO_DIV, STOP.STOP_HLT, \
+             STOP.STOP_SYSCALL_ARM, STOP.STOP_X86_CPUID}
+
+    def __getstate__(self):
+        parent_ret = super().__getstate__()
+        return (parent_ret, self._block_details_cache, self._stop_block_addrs_cache, self._stop_reasons_to_track)
+
+    def __setstate__(self, args):
+        super().__setstate__(args[0])
+        self._block_details_cache = args[1]
+        self._stop_block_addrs_cache = args[2]
+        self._stop_reasons_to_track = args[3]
+
     def __check(self, num_inst=None, **kwargs):  # pylint: disable=unused-argument
         state = self.state
         if o.UNICORN not in state.options:
@@ -75,6 +97,10 @@ class SimEngineUnicorn(SuccessorsMixin):
             l.info("failed register check")
             return False
 
+        if state.addr in self._stop_block_addrs_cache:
+            l.info("Block will likely not execute in native interface")
+            return False
+
         return True
 
     @staticmethod
@@ -84,19 +110,12 @@ class SimEngineUnicorn(SuccessorsMixin):
         state.unicorn.countdown_unsupported_stop -= 1
         state.unicorn.countdown_stop_point -= 1
 
-    @staticmethod
-    def __reset_countdowns(state, next_state):
-        next_state.unicorn.countdown_symbolic_stop = 0
-        next_state.unicorn.countdown_unsupported_stop = 0
-        next_state.unicorn.countdown_nonunicorn_blocks = state.unicorn.countdown_nonunicorn_blocks
-        next_state.unicorn.countdown_stop_point = state.unicorn.countdown_stop_point
-
     def _execute_block_instrs_in_vex(self, block_details):
-        if block_details["block_addr"] not in self.block_details_cache:
+        if block_details["block_addr"] not in self._block_details_cache:
             vex_block = self._get_vex_block_details(block_details["block_addr"], block_details["block_size"])
-            self.block_details_cache[block_details["block_addr"]] = vex_block
+            self._block_details_cache[block_details["block_addr"]] = vex_block
         else:
-            vex_block = self.block_details_cache[block_details["block_addr"]]
+            vex_block = self._block_details_cache[block_details["block_addr"]]
 
         # Save breakpoints for restoring later
         saved_mem_read_breakpoints = copy.copy(self.state.inspect._breakpoints["mem_read"])
@@ -135,6 +154,7 @@ class SimEngineUnicorn(SuccessorsMixin):
         # Restore breakpoints
         for succ_state in self.successors.successors:
             succ_state.inspect._breakpoints["mem_read"] = copy.copy(saved_mem_read_breakpoints)
+            succ_state.inspect._breakpoints["mem_write"] = copy.copy(saved_mem_read_breakpoints)
 
         del self.stmt_idx
 
@@ -289,26 +309,48 @@ class SimEngineUnicorn(SuccessorsMixin):
                 state.inspect.mem_read_expr = state.solver.BVV(value)
         else:
             # The value may be partially concrete. Set the symbolic bitmap to read correct value and restore it
-            mem_read_address = state.inspect.mem_read_address
-            mem_read_length = state.inspect.mem_read_length
+            mem_read_addr = state.solver.eval(state.inspect.mem_read_address)
+            mem_read_len = state.inspect.mem_read_length
             saved_taints = []
-            # Save current symbolic taint bitmap and set it to what read expects
-            for offset, taint in enumerate(taint_map):
-                page_num, page_off = state.memory._divide_addr(state.solver.eval(mem_read_address) + offset)
-                saved_taints.append(state.memory._pages[page_num].symbolic_bitmap[page_off])
-                if taint != -1:
-                    # Current taint may not be correct so update it.
-                    state.memory._pages[page_num].symbolic_bitmap[page_off] = taint
+            for offset in range(mem_read_len):
+                page_num, page_off = state.memory._divide_addr(mem_read_addr + offset)
+                page_obj = state.memory._get_page(page_num, writing=False)
+                saved_taints.append(page_obj.symbolic_bitmap[page_off])
 
-            # Perform memory read
-            state.inspect.mem_read_expr = state.memory.load(mem_read_address, mem_read_length,
-                                                            endness=state.arch.memory_endness, inspect=False,
-                                                            disable_actions=True)
+            restore_taints = False
+            if saved_taints != taint_map:
+                # Symbolic bitmap needs fixing before reading value from memory.
+                restore_taints = True
+                for offset, expected_taint in enumerate(taint_map):
+                    if expected_taint != -1:
+                        page_num, page_off = state.memory._divide_addr(mem_read_addr + offset)
+                        page_obj = state.memory._get_page(page_num, writing=False)
+                        page_obj.symbolic_bitmap[page_off] = expected_taint
 
-            # Restore symbolic taint bitmap
-            for offset, saved_taint in enumerate(saved_taints):
-                page_num, page_off = state.memory._divide_addr(state.solver.eval(mem_read_address) + offset)
-                state.memory._pages[page_num].symbolic_bitmap[page_off] = saved_taint
+            curr_value = state.memory.load(mem_read_addr, mem_read_len, endness=state.arch.memory_endness,
+                                           inspect=False, disable_actions=True)
+            if restore_taints:
+                for offset, saved_taint in enumerate(saved_taints):
+                    page_num, page_off = state.memory._divide_addr(mem_read_addr + offset)
+                    page_obj = state.memory._get_page(page_num, writing=False)
+                    page_obj.symbolic_bitmap[page_off] = saved_taint
+
+            if taint_map.count(0) != 0:
+                # Update concrete bytes using values reported by native interface
+                curr_value_bytes = curr_value.chop(8)
+                if state.arch.memory_endness == archinfo.Endness.LE:
+                    curr_value_bytes.reverse()
+
+                for offset, expected_taint in enumerate(taint_map):
+                    if expected_taint == 0:
+                        curr_value_bytes[offset] = state.solver.BVV(value[offset], 8)
+
+                if state.arch.memory_endness == archinfo.Endness.LE:
+                    curr_value_bytes = reversed(curr_value_bytes)
+
+                curr_value = state.solver.Concat(*curr_value_bytes)
+
+            state.inspect.mem_read_expr = curr_value
 
     def _save_mem_write_addrs(self, state):
         mem_write_addr = state.solver.eval(state.inspect.mem_write_address)
@@ -348,9 +390,6 @@ class SimEngineUnicorn(SuccessorsMixin):
                 # will then be handled by another engine that can more accurately step instruction-by-instruction.
                 extra_stop_points.add(bp.kwargs["instruction"])
 
-        # VEX block cache for executing instructions skipped by native interface
-        self.block_details_cache = {}  # pylint:disable=attribute-defined-outside-init
-
         # initialize unicorn plugin
         try:
             syscall_data = kwargs["syscall_data"] if "syscall_data" in kwargs else None
@@ -359,7 +398,6 @@ class SimEngineUnicorn(SuccessorsMixin):
         except SimValueError:
             # it's trying to set a symbolic register somehow
             # fail out, force fallback to next engine
-            self.__reset_countdowns(successors.initial_state, state)
             return super().process_successors(successors, **kwargs)
 
         try:
@@ -376,10 +414,14 @@ class SimEngineUnicorn(SuccessorsMixin):
             state.unicorn.destroy(self.state)
 
         state = self.state
+        if state.unicorn.stop_reason in self._stop_reasons_to_track:
+            if state.unicorn.steps == 0:
+                self._stop_block_addrs_cache.add(state.addr)
+            else:
+                self._stop_block_addrs_cache.add(state.unicorn.stop_details.block_addr)
 
         if state.unicorn.steps == 0 or state.unicorn.stop_reason == STOP.STOP_NOSTART:
             # fail out, force fallback to next engine
-            self.__reset_countdowns(successors.initial_state, state)
             # TODO: idk what the consequences of this might be. If this failed step can actually change non-unicorn
             # state then this is bad news.
             return super().process_successors(successors, **kwargs)
@@ -397,7 +439,7 @@ class SimEngineUnicorn(SuccessorsMixin):
                         state._inspect('irsb', BP_AFTER, address=bbl_addr)
                     break
 
-        if (state.unicorn.stop_reason in (STOP.symbolic_stop_reasons + STOP.unsupported_reasons) or
+        if (state.unicorn.stop_reason in (STOP.symbolic_stop_reasons | STOP.unsupported_reasons) or
             state.unicorn.stop_reason in (STOP.STOP_UNKNOWN_MEMORY_WRITE_SIZE, STOP.STOP_VEX_LIFT_FAILED)):
             l.info(state.unicorn.stop_message)
 
