@@ -1,3 +1,5 @@
+# pylint:disable=missing-class-docstring
+
 class AngrError(Exception):
     pass
 
@@ -315,6 +317,9 @@ class UnsupportedIRStmtError(SimStatementError, SimUnsupportedError):
 class UnsupportedDirtyError(UnsupportedIRStmtError, SimUnsupportedError):
     pass
 
+class SimMissingTempError(SimValueError, IndexError):
+    pass
+
 #
 # Engine-related errors
 #
