@@ -1,8 +1,31 @@
+import random
 from collections import UserDict
 
 from .plugin import KnowledgeBasePlugin
 from ..sim_type import ALL_TYPES, TypeRef
 
+
+FRUITS = [
+    'mango',
+    'cherry',
+    'banana',
+    'papaya',
+    'apple',
+    'kiwi',
+    'pineapple',
+    'coconut',
+    'peach',
+    'honeydew',
+    'cucumber',
+    'pumpkin',
+    'cantaloupe',
+    'strawberry',
+    'watermelon',
+    'nectarine',
+    'orange',
+]
+
+
 class TypesStore(KnowledgeBasePlugin, UserDict):
     """
     A kb plugin that stores a mapping from name to TypeRef. It will return types from angr.sim_type.ALL_TYPES as
@@ -45,4 +68,14 @@ class TypesStore(KnowledgeBasePlugin, UserDict):
         value._name = new
         self[new] = value
 
+    def unique_type_name(self) -> str:
+        for fruit in FRUITS:
+            if fruit not in self:
+                name = fruit
+                break
+        else:
+            name = f'type_{random.randint(0x10000000, 0x100000000):x}'
+        return name
+
+
 KnowledgeBasePlugin.register_default('types', TypesStore)
