@@ -399,25 +399,28 @@ class VariableManagerInternal(Serializable):
                     existing_phis.add(phivar)
 
         if len(existing_phis) >= 1:
-            existing_phi = next(iter(existing_phis))
-            if block_addr in self._phi_variables_by_block and existing_phi in self._phi_variables_by_block[block_addr]:
-                if not non_phis.issubset(self.get_phi_subvariables(existing_phi)):
-                    # Update the variables that this phi variable represents
-                    self._phi_variables[existing_phi] |= non_phis
-                return existing_phi
+            # iterate through existing phi variables to see if any of it is already used as the phi variable for this
+            # block. if so, we reuse it to avoid redundant variable allocations
+            for phi in existing_phis:
+                if block_addr in self._phi_variables_by_block and phi in self._phi_variables_by_block[block_addr]:
+                    if not non_phis.issubset(self.get_phi_subvariables(phi)):
+                        # Update the variables that this phi variable represents
+                        self._phi_variables[phi] |= non_phis
+                    return phi
 
         # allocate a new phi variable
         repre = next(iter(variables))
         repre_type = type(repre)
+        repre_size = max(var.size for var in variables)
         if repre_type is SimRegisterVariable:
             ident_sort = 'register'
-            a = SimRegisterVariable(repre.reg, repre.size, ident=self.next_variable_ident(ident_sort))
+            a = SimRegisterVariable(repre.reg, repre_size, ident=self.next_variable_ident(ident_sort))
         elif repre_type is SimMemoryVariable:
             ident_sort = 'global'
-            a = SimMemoryVariable(repre.addr, repre.size, ident=self.next_variable_ident(ident_sort))
+            a = SimMemoryVariable(repre.addr, repre_size, ident=self.next_variable_ident(ident_sort))
         elif repre_type is SimStackVariable:
             ident_sort = 'stack'
-            a = SimStackVariable(repre.offset, repre.size, ident=self.next_variable_ident(ident_sort))
+            a = SimStackVariable(repre.offset, repre_size, ident=self.next_variable_ident(ident_sort))
         else:
             raise TypeError('make_phi_node(): Unsupported variable type "%s".' % type(repre))
 
@@ -658,6 +661,9 @@ class VariableManagerInternal(Serializable):
         :return:        None
         """
 
+        def _id_from_varident(ident: str) -> int:
+            return int(ident[ident.find("_") + 1:])
+
         if not self._unified_variables:
             return
 
@@ -667,7 +673,7 @@ class VariableManagerInternal(Serializable):
 
         for var in self._unified_variables:
             if isinstance(var, SimStackVariable):
-                if var.ident and var.ident.startswith('iarg_'):
+                if var.ident and var.ident.startswith('arg_'):
                     arg_vars.append(var)
                 else:
                     sorted_stack_variables.append(var)
@@ -695,7 +701,7 @@ class VariableManagerInternal(Serializable):
         var_ctr = count(0)
 
         sorted_stack_variables = sorted(sorted_stack_variables, key=lambda v: v.offset)
-        sorted_reg_variables = sorted(sorted_reg_variables, key=lambda v: v.reg)
+        sorted_reg_variables = sorted(sorted_reg_variables, key=lambda v: _id_from_varident(v.ident))
 
         for var in chain(sorted_stack_variables, sorted_reg_variables):
             idx = next(var_ctr)
@@ -710,7 +716,7 @@ class VariableManagerInternal(Serializable):
 
         # rename arguments but keeping the original order
         arg_ctr = count(0)
-        arg_vars = sorted(arg_vars, key=lambda v: int(v.ident[v.ident.index("_")+1:]) if v.ident else 0)
+        arg_vars = sorted(arg_vars, key=lambda v: _id_from_varident(v.ident))
         for var in arg_vars:
             idx = next(arg_ctr)
             if var.name is not None and not reset:
@@ -787,17 +793,17 @@ class VariableManagerInternal(Serializable):
                 self.set_unified_variable(v, unified)
 
         # unify register variables based on phi nodes
-        graph = networkx.Graph()
+        graph = networkx.DiGraph()  # an edge v1 -> v2 means v2 is the phi variable for v1
         for v, subvs in self._phi_variables.items():
             if not isinstance(v, SimRegisterVariable):
                 continue
             for subv in subvs:
-                graph.add_edge(v, subv)
+                graph.add_edge(subv, v)
 
         # prune the graph: remove nodes that have never been used
         while True:
             unused_nodes = set()
-            for node in [ nn for nn in graph.nodes() if graph.degree[nn] == 1]:
+            for node in [ nn for nn in graph.nodes() if graph.out_degree[nn] == 0]:
                 if not self.get_variable_accesses(node):
                     # this node has never been used - discard it
                     unused_nodes.add(node)
@@ -806,7 +812,12 @@ class VariableManagerInternal(Serializable):
             else:
                 break
 
-        for nodes in networkx.connected_components(graph):
+        # convert the directional graph into a non-directional graph
+        graph_ = networkx.Graph()
+        graph_.add_nodes_from(graph.nodes)
+        graph_.add_edges_from(graph.edges)
+
+        for nodes in networkx.connected_components(graph_):
             if len(nodes) <= 1:
                 continue
             nodes = list(nodes)
