@@ -1,4 +1,3 @@
-import claripy
 import angr
 from angr.sim_options import MEMORY_CHUNK_INDIVIDUAL_READS
 from angr.storage.memory_mixins.regioned_memory.abstract_address_descriptor import AbstractAddressDescriptor
@@ -10,14 +9,18 @@ class strlen(angr.SimProcedure):
     #pylint:disable=arguments-differ
     max_null_index = None
 
-    def run(self, s, wchar=False):
+    def run(self, s, wchar=False, maxlen=None):
         if wchar:
             null_seq = self.state.solver.BVV(0, 16)
+            char_size = 2
         else:
             null_seq = self.state.solver.BVV(0, 8)
+            char_size = 1
 
         max_symbolic_bytes = self.state.libc.buf_symbolic_bytes
         max_str_len = self.state.libc.max_str_len
+        if maxlen:
+            max_str_len = min(maxlen, max_str_len)
 
         chunk_size = None
         if MEMORY_CHUNK_INDIVIDUAL_READS in self.state.options:
@@ -30,11 +33,12 @@ class strlen(angr.SimProcedure):
             # Make sure to convert s to ValueSet
             addr_desc: AbstractAddressDescriptor = self.state.memory._normalize_address(s)
 
-            length = self.state.solver.ESI(self.state.arch.bits)
+            # size_t
+            length = self.state.solver.ESI(self.arch.bits)
             for s_aw in self.state.memory._concretize_address_descriptor(addr_desc, None):
 
                 s_ptr = s_aw.to_valueset(self.state)
-                r, c, i = self.state.memory.find(s, null_seq, max_str_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size)
+                r, c, i = self.state.memory.find(s, null_seq, max_str_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size, char_size=char_size)
 
                 self.max_null_index = max([self.max_null_index] + i)
 
@@ -50,14 +54,14 @@ class strlen(angr.SimProcedure):
 
         else:
             search_len = max_str_len
-            r, c, i = self.state.memory.find(s, null_seq, search_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size)
+            r, c, i = self.state.memory.find(s, null_seq, search_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size, char_size=char_size)
 
             # try doubling the search len and searching again
             s_new = s
             while c and all(con.is_false() for con in c):
                 s_new += search_len
                 search_len *= 2
-                r, c, i = self.state.memory.find(s_new, null_seq, search_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size)
+                r, c, i = self.state.memory.find(s_new, null_seq, search_len, max_symbolic_bytes=max_symbolic_bytes, chunk_size=chunk_size, char_size=char_size)
                 # stop searching after some reasonable limit
                 if search_len > 0x10000:
                     raise angr.SimMemoryLimitError("strlen hit limit of 0x10000")
@@ -66,7 +70,7 @@ class strlen(angr.SimProcedure):
             self.state.add_constraints(*c)
             result = r - s
             if result.depth > 3:
-                rresult = claripy.BVS('strlen', len(result))
-                self.state.solver.add(result == rresult)
+                rresult = self.state.solver.BVS('strlen', len(result), key=('api', 'strlen'))
+                self.state.add_constraints(result == rresult)
                 result = rresult
             return result
