@@ -178,6 +178,9 @@ UNICORN_HANDLE_SYMBOLIC_ADDRESSES = "UNICORN_HANDLE_SYMBOLIC_ADDRESSES"
 # Indicates if symbolic conditions should be handled in unicorn or not
 UNICORN_HANDLE_SYMBOLIC_CONDITIONS = "UNICORN_HANDLE_SYMBOLIC_CONDITIONS"
 
+# Handle syscalls in unicorn even if arguments are symbolic
+UNICORN_HANDLE_SYMBOLIC_SYSCALLS = "UNICORN_HANDLE_SYMBOLIC_SYSCALLS"
+
 # floating point support
 SUPPORT_FLOATING_POINT = "SUPPORT_FLOATING_POINT"
 
