@@ -20,6 +20,7 @@ from .misc.plugins import PluginHub, PluginPreset
 from .sim_state_options import SimStateOptions
 from .state_plugins import SimStatePlugin
 
+
 def arch_overrideable(f):
     @functools.wraps(f)
     def wrapped_f(self, *args, **kwargs):
@@ -71,6 +72,7 @@ class SimState(PluginHub):
     history: 'SimStateHistory'
     inspect: 'SimInspector'
     jni_references: "SimStateJNIReferences"
+    scratch: "SimStateScratch"
     def __init__(
             self,
             project=None,
@@ -385,7 +387,7 @@ class SimState(PluginHub):
     T = TypeVar("T")
     def _inspect_getattr(self, attr: str, default_value: T):
         if self.supports_inspect:
-            if self.inspect.action_attrs_set and hasattr(self.inspect, attr):
+            if hasattr(self.inspect, attr):
                 return getattr(self.inspect, attr)
 
         return default_value
@@ -984,4 +986,5 @@ if TYPE_CHECKING:
     from .state_plugins.view import SimRegNameView, SimMemView
     from .state_plugins.callstack import CallStack
     from .state_plugins.inspect import SimInspector
-    from .state_plugins import SimStateJNIReferences
+    from .state_plugins.jni_references import SimStateJNIReferences
+    from .state_plugins.scratch import SimStateScratch
