@@ -248,31 +248,41 @@ class SimInspector(SimStatePlugin):
         for t in event_types:
             self._breakpoints[t] = [ ]
 
+        self.action_attrs_set = False  # action() will set it to True if the kwargs passed in have been set as
+                                       # attributes to self.
+
         for i in inspect_attributes:
             setattr(self, i, None)
 
     def __dir__(self):
         return sorted(set(dir(super(SimInspector, self)) + dir(inspect_attributes) + dir(self.__class__)))
 
-    def action(self, event_type, when, **kwargs):
-        """
-        Called from within SimuVEX when events happens. This function checks all breakpoints registered for that event
-        and fires the ones whose conditions match.
-        """
-        l.debug("Event %s (%s) firing...", event_type, when)
+    def _set_inspect_attrs(self, **kwargs):
         for k,v in kwargs.items():
             if k not in inspect_attributes:
-                raise ValueError("Invalid inspect attribute %s passed in. Should be one of: %s" % (k, inspect_attributes))
-            #l.debug("... %s = %r", k, v)
-            l.debug("... setting %s", k)
+                raise ValueError("Invalid inspect attribute %s passed in. Should be one of: %s" %
+                                 (k, inspect_attributes))
+            # l.debug("... setting %s", k)
             setattr(self, k, v)
 
+    def action(self, event_type, when, **kwargs):
+        """
+        Called from within the engine when events happens. This function checks all breakpoints registered for that
+        event and fires the ones whose conditions match.
+        """
+
+        self._set_inspect_attrs(**kwargs)
+        self.action_attrs_set = True
+
         for bp in self._breakpoints[event_type]:
-            l.debug("... checking bp %r", bp)
+            if not self.action_attrs_set:
+                self._set_inspect_attrs(**kwargs)
+                self.action_attrs_set = True
             if bp.check(self.state, when):
-                l.debug("... FIRE")
                 bp.fire(self.state)
 
+        self.action_attrs_set = False
+
     def make_breakpoint(self, event_type, *args, **kwargs):
         """
         Creates and adds a breakpoint which would trigger on `event_type`. Additional arguments are passed to the
