@@ -48,6 +48,7 @@ class TRANSMIT_RECORD(ctypes.Structure):
     """
 
     _fields_ = [
+        ('fd', ctypes.c_uint32),
         ('data', ctypes.c_void_p),
         ('count', ctypes.c_uint32)
     ]
@@ -184,13 +185,13 @@ class STOP:
     stop_message[STOP_SYSCALL_ARM]   = "ARM syscalls are currently not supported by SimEngineUnicorn"
     stop_message[STOP_X86_CPUID] = "Block executes cpuid which should be handled in VEX engine"
 
-    symbolic_stop_reasons = [STOP_SYMBOLIC_PC, STOP_SYMBOLIC_READ_ADDR, STOP_SYMBOLIC_READ_SYMBOLIC_TRACKING_DISABLED,
-        STOP_SYMBOLIC_WRITE_ADDR, STOP_SYMBOLIC_BLOCK_EXIT_CONDITION, STOP_SYMBOLIC_BLOCK_EXIT_TARGET, STOP_SYSCALL_ARM,
-        STOP_X86_CPUID]
+    symbolic_stop_reasons = {STOP_SYMBOLIC_PC, STOP_SYMBOLIC_READ_ADDR, STOP_SYMBOLIC_READ_SYMBOLIC_TRACKING_DISABLED,
+        STOP_SYMBOLIC_WRITE_ADDR, STOP_SYMBOLIC_BLOCK_EXIT_CONDITION, STOP_SYMBOLIC_BLOCK_EXIT_TARGET,
+        STOP_SYSCALL_ARM, STOP_X86_CPUID}
 
-    unsupported_reasons = [STOP_UNSUPPORTED_STMT_PUTI, STOP_UNSUPPORTED_STMT_STOREG, STOP_UNSUPPORTED_STMT_LOADG,
+    unsupported_reasons = {STOP_UNSUPPORTED_STMT_PUTI, STOP_UNSUPPORTED_STMT_STOREG, STOP_UNSUPPORTED_STMT_LOADG,
         STOP_UNSUPPORTED_STMT_CAS, STOP_UNSUPPORTED_STMT_LLSC, STOP_UNSUPPORTED_STMT_DIRTY,
-        STOP_UNSUPPORTED_STMT_UNKNOWN, STOP_UNSUPPORTED_EXPR_UNKNOWN, STOP_VEX_LIFT_FAILED]
+        STOP_UNSUPPORTED_STMT_UNKNOWN, STOP_UNSUPPORTED_EXPR_UNKNOWN, STOP_VEX_LIFT_FAILED}
 
     @staticmethod
     def name_stop(num):
@@ -402,7 +403,7 @@ def _load_native():
 
         #_setup_prototype_explicit(h, 'logSetLogLevel', None, ctypes.c_uint64)
         _setup_prototype(h, 'alloc', state_t, uc_engine_t, ctypes.c_uint64, ctypes.c_uint64, ctypes.c_bool,
-                         ctypes.c_bool)
+                         ctypes.c_bool, ctypes.c_bool)
         _setup_prototype(h, 'dealloc', None, state_t)
         _setup_prototype(h, 'hook', None, state_t)
         _setup_prototype(h, 'unhook', None, state_t)
@@ -1100,7 +1101,9 @@ class Unicorn(SimStatePlugin):
         # tricky: using unicorn handle from unicorn.Uc object
         handle_symb_addrs = options.UNICORN_HANDLE_SYMBOLIC_ADDRESSES in self.state.options
         handle_symb_conds = options.UNICORN_HANDLE_SYMBOLIC_CONDITIONS in self.state.options
-        self._uc_state = _UC_NATIVE.alloc(self.uc._uch, self.cache_key, simos_val, handle_symb_addrs, handle_symb_conds)
+        handle_symbolic_syscalls = options.UNICORN_HANDLE_SYMBOLIC_SYSCALLS in self.state.options
+        self._uc_state = _UC_NATIVE.alloc(self.uc._uch, self.cache_key, simos_val, handle_symb_addrs, handle_symb_conds,
+                                          handle_symbolic_syscalls)
 
         if options.UNICORN_SYM_REGS_SUPPORT in self.state.options and \
                 options.UNICORN_AGGRESSIVE_CONCRETIZATION not in self.state.options:
@@ -1268,7 +1271,7 @@ class Unicorn(SimStatePlugin):
         unicorn_obj.stop_details = _UC_NATIVE.get_stop_details(self._uc_state)
         unicorn_obj.stop_reason = unicorn_obj.stop_details.stop_reason
         unicorn_obj.stop_message = STOP.get_stop_msg(unicorn_obj.stop_reason)
-        if unicorn_obj.stop_reason in (STOP.symbolic_stop_reasons + STOP.unsupported_reasons) or \
+        if unicorn_obj.stop_reason in (STOP.symbolic_stop_reasons | STOP.unsupported_reasons) or \
           unicorn_obj.stop_reason in {STOP.STOP_UNKNOWN_MEMORY_WRITE_SIZE, STOP.STOP_VEX_LIFT_FAILED}:
             stop_block_addr = unicorn_obj.stop_details.block_addr
             stop_block_size = unicorn_obj.stop_details.block_size
@@ -1308,6 +1311,7 @@ class Unicorn(SimStatePlugin):
         # process the concrete transmits
         i = 0
         stdout = state.posix.get_fd(1)
+        stderr = state.posix.get_fd(2)
 
         while True:
             record = _UC_NATIVE.process_transmit(self._uc_state, i)
@@ -1315,7 +1319,10 @@ class Unicorn(SimStatePlugin):
                 break
 
             string = ctypes.string_at(record.contents.data, record.contents.count)
-            stdout.write_data(string)
+            if record.contents.fd == 1:
+                stdout.write_data(string)
+            elif record.contents.fd == 2:
+                stderr.write_data(string)
             i += 1
 
         # Re-execute concrete writes
