@@ -3,6 +3,8 @@ from .timing import timethis
 from . import graph
 from . import constants
 from . import enums_conv
+from . import lazy_import
+from .env import is_pyinstaller
 
 
 def looks_like_sql(s: str) -> bool:
@@ -18,8 +20,14 @@ def looks_like_sql(s: str) -> bool:
     s = s.lower()
     for k in sql_keywords:
         if k in s:
+            k_index = s.find(k)
+            # what's before k? is it a whitespace if it's not empty?
+            if k_index > 0:
+                before = s[k_index - 1]
+                if before not in " /;":
+                    continue
             # what's after k? is it a whitespace?
-            following = s[s.find(k) + len(k) : ]
+            following = s[k_index + len(k) : ]
             if not following or following.startswith(" "):
                 return True
 
